{ This demo program shows how to use the matrix/vector functions implemented
  in ChroMathsome.
  This code was (is) written during the development stages of ChroMathsome.
  There's no pre-written script followed so you may find (some of) this code
  (the order of implemented functions) a little strange.
  The unit Algebra.dcu is not fully debugged (16 september '96).
  This file has been added to the package to give interested people a preview of what
  algebra.dcu is capable of and how easy it is to use.
  When it is completed, the finished version will be uploaded to the same site you
  found this file.
  If you compile this program, the resulting Linalgeb.exe will work just fine.
  DO NOT ALTER THE CODE BELOW.
  If you ignore this message and everything screws up, that's your responsibility not
  mine!
  Besides, Algebra.dcu contains more than twice the functions than the ones shown here
  (take a look at the size of Algebra.dcu) so you'll never get the full potential of
  what is available.

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write super sofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).
  Note also that the functions below perform their actions on random generated
  matrices and vectors.
  This has the benefit that almost all functions can be fully tested (exceptions!) at a hugh tempo.
  There is no user interface (except for the menus) written in this program.

  Most functions work for matrices up to 20*20 (and 20 dimensional vectors).
  To avoid a constant 'invalid dimension' error (it's all random data ...) we use a
  lower value so that there are possibilities that the dimensions fit (ex. adding two same dimesional matrices).

  Author: John Merrit (john.merrit@net4all.be)
  Release date: 18 september 1996

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

                }

unit Matrices;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Algebra, Calculus, Menus;

type
  TMainForm = class(TForm)
    MathMemo: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Matrix1: TMenuItem;
    Random1: TMenuItem;
    RealMatrix1: TMenuItem;
    ComplexMatrix1: TMenuItem;
    Arithmetic1: TMenuItem;
    RealMatrix2: TMenuItem;
    Addition1: TMenuItem;
    Subtraction1: TMenuItem;
    Multiplication1: TMenuItem;
    ComplexMatrix2: TMenuItem;
    Addition2: TMenuItem;
    Subtraction2: TMenuItem;
    Multiplication2: TMenuItem;
    Functions1: TMenuItem;
    RealMatrix3: TMenuItem;
    Trace1: TMenuItem;
    Determinant1: TMenuItem;
    Transpose1: TMenuItem;
    Inverse1: TMenuItem;
    Rank1: TMenuItem;
    ComplexMatrix3: TMenuItem;
    Trace2: TMenuItem;
    Determinant2: TMenuItem;
    Transpose2: TMenuItem;
    Inverse2: TMenuItem;
    Rank2: TMenuItem;
    Reduction1: TMenuItem;
    RealMatrix4: TMenuItem;
    ComplexMatrix4: TMenuItem;
    Gauss1: TMenuItem;
    Hermite1: TMenuItem;
    Gauss2: TMenuItem;
    Hermite2: TMenuItem;
    Vector1: TMenuItem;
    Random2: TMenuItem;
    RealVector1: TMenuItem;
    ComplexVector1: TMenuItem;
    ClearMemo1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Create1: TMenuItem;
    RealMatrix5: TMenuItem;
    ComplexMatrix5: TMenuItem;
    AddRow1: TMenuItem;
    AddColumn1: TMenuItem;
    DeleteRow1: TMenuItem;
    DeleteColumn1: TMenuItem;
    SwapRows1: TMenuItem;
    SwapColumns1: TMenuItem;
    InsertRow1: TMenuItem;
    InsertColumn1: TMenuItem;
    ReplaceRow1: TMenuItem;
    ReplaceColumn1: TMenuItem;
    MatrixVector1: TMenuItem;
    VectorMatrix1: TMenuItem;
    MatrixRowVector1: TMenuItem;
    MatrixColVector1: TMenuItem;
    N2: TMenuItem;
    SpecialMatrices1: TMenuItem;
    RealMatrix6: TMenuItem;
    ComplexMatrix6: TMenuItem;
    Identity1: TMenuItem;
    Hilbert1: TMenuItem;
    Vandermonde1: TMenuItem;
    Householder1: TMenuItem;
    Identity2: TMenuItem;
    Hilbert2: TMenuItem;
    Vandermonde2: TMenuItem;
    Householder2: TMenuItem;
    Fourier1: TMenuItem;
    Factorization1: TMenuItem;
    RealMatrix7: TMenuItem;
    ComplexMatrix7: TMenuItem;
    LU1: TMenuItem;
    QR1: TMenuItem;
    OrthogonalizesetofVectors1: TMenuItem;
    N3: TMenuItem;
    CharacteristicEquation1: TMenuItem;
    ToPower1: TMenuItem;
    ToPower2: TMenuItem;
    N4: TMenuItem;
    SysofEquations1: TMenuItem;
    ReplaceMatrixElement1: TMenuItem;
    Functions2: TMenuItem;
    Arithmetic2: TMenuItem;
    RealVector2: TMenuItem;
    ComplexVector2: TMenuItem;
    Norm1: TMenuItem;
    UnitVector1: TMenuItem;
    Norm2: TMenuItem;
    UnitVector2: TMenuItem;
    RealVector3: TMenuItem;
    ComplexVector3: TMenuItem;
    Addition3: TMenuItem;
    Subtraction3: TMenuItem;
    Scalardotproduct1: TMenuItem;
    Vectorcrossproduct1: TMenuItem;
    Angle1: TMenuItem;
    Distance1: TMenuItem;
    procedure ClearMemo1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure RandomMatrix(Sender: TObject);
    procedure RealArithmetic(Sender: TObject);
    procedure ComplexArithmetic(Sender: TObject);
    procedure RealFunction(Sender: TObject);
    procedure ComplexFunction(Sender: TObject);
    procedure RealReduction(Sender: TObject);
    procedure ComplexReduction(Sender: TObject);
    procedure CreateRealMatrix(Sender: TObject);
    procedure SpecialRealMatrix(Sender: TObject);
    procedure SpecialComplexMatrix(Sender: TObject);
    procedure RealMatrixFactorization(Sender: TObject);
    procedure CharacteristicEquation1Click(Sender: TObject);
    procedure SysofEquations1Click(Sender: TObject);
    procedure RealVector1Click(Sender: TObject);
    procedure Norm1Click(Sender: TObject);
    procedure Norm2Click(Sender: TObject);
    procedure RealVectorFunction(Sender: TObject);
    
  private
    { Private declarations }
    procedure Random_RealMatrix(var M: RealMatrix);
    procedure Random_ComplexMatrix(var M: ComplexMatrix);
    procedure Random_RealVector(var V: RealVector);
    procedure Random_ComplexVector(var V: ComplexVector);
    procedure Write_Message(s: string);
    procedure Write_RealMatrix(M: RealMatrix);
    procedure Write_ComplexMatrix(M: ComplexMatrix);
    procedure Write_RealVector(V: RealVector);
    procedure Write_ComplexVector(V: ComplexVector);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}


procedure TMainForm.Write_Message(s: string);
begin
     MathMemo.Lines.add(s);
end;

{ Write a real matrix to the screen, creating a nice readable look. }
procedure TMainForm.Write_RealMatrix(M: RealMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    MathMemo.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if int(M.Matrix[a,b]) = M.Matrix[a,b] then
    matt:= IntToStr(round(M.Matrix[a,b]))
    else
    begin
    if abs(M.Matrix[a,b]) < 1E-10 then
    begin
    M.Matrix[a,b]:= 0;
    matt:= IntToStr(round(M.Matrix[a,b]));
    end
    else
    matt:= FloatToStrF(M.Matrix[a,b], ffexponent,10,2);
    end;
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    MathMemo.Lines.add(mats);
    end;
end;

{ Complex counterpart of above. }
procedure TMainForm.Write_ComplexMatrix(M: ComplexMatrix);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
begin
    MathMemo.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix[a,b].re;
    if abs(c) < 1E-12 then c:= 0;
    d:= M.Matrix[a,b].im;
    if abs(d) < 1E-12 then d:= 0;
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    MathMemo.Lines.add(mats);
    end;
end;

{ Write a real vector to the screen, creating a nice readable look. }
procedure TMainForm.Write_RealVector(V: RealVector);
var a:integer;
    matt,mats,mattt:string;
begin
    MathMemo.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    if int(V.Vector[a]) = V.Vector[a] then
    matt:= IntToStr(round(V.Vector[a]))
    else
    begin
    if abs(V.Vector[a]) < 1E-10 then
    begin
    V.Vector[a]:= 0;
    matt:= IntToStr(round(V.Vector[a]));
    end
    else
    matt:= FloatToStrF(V.Vector[a], ffexponent,5,2);
    end;
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    MathMemo.Lines.add(mats);
end;

{ Complex counterpart of above. }
procedure TMainForm.Write_ComplexVector(V: ComplexVector);
var a:integer;
    matt,mats,mattt:string;
begin
    MathMemo.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    matt:= '(' + FloatToStr(V.Vector[a].re) + ';' +  FloatToStr(V.Vector[a].im) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    MathMemo.Lines.add(mats);
end;

{ Clear the memo. }
procedure TMainForm.ClearMemo1Click(Sender: TObject);
begin
    if MessageDlg('Delete the contents of this memo?',
    mtInformation, [mbYes, mbNo], 0) = mrYes
    then
    MathMemo.Lines.Clear;
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
    Close;
end;

{ Generate a random real matrix. We'll create a square matrix so that almost all functions will
  work without the "Invalid matrix dimensions!" error occuring too often.                       }
procedure TMainForm.Random_RealMatrix(var M: RealMatrix);
var i,j: integer;
begin
    i:= random(15) + 1;
    M.Row:= i;
    M.Col:= i;
    RRandom_Matrix(M, i, i, 5);
end;

{ Complex counterpart of above. }
procedure TMainForm.Random_ComplexMatrix(var M: ComplexMatrix);
var i: integer;
begin
    i:= random(15);
    CRandom_Matrix(M, i, i, 100);
end;

{ A random real vector. }
procedure TMainForm.Random_RealVector(var V: RealVector);
var i: integer;
begin
    i:= Random(5) + 1;
    RRandom_Vector(V, i, 100);
end;

{ Complex counterpart of above. }
procedure TMainForm.Random_ComplexVector(var V: ComplexVector);
var i: integer;
begin
    i:= Random(15) + 1;
    CRandom_Vector(V, i, 100);
end;

procedure TMainForm.RandomMatrix(Sender: TObject);
var RM: ^RealMatrix;
    CM: ^ComplexMatrix;
begin
     new(RM);
     new(CM);
     try
     if Sender = RealMatrix1 then
     begin
     Write_Message('A random real matrix:');
     Random_RealMatrix(RM^);
     Write_RealMatrix(RM^);
     end
     else
     begin
     Write_Message('A random complex matrix:');
     Random_ComplexMatrix(CM^);
     Write_ComplexMatrix(CM^);
     end;
     finally
     dispose(RM);
     dispose(CM);
     end;
end;

{ Arithmetic with real matrices. }
procedure TMainForm.RealArithmetic(Sender: TObject);
var M1, M2, MResult: ^RealMatrix;
    i: integer;
begin
     new(M1);
     new(M2);
     new(MResult);
     try
     Write_Message('A random real matrix:');
     Random_RealMatrix(M1^);
     Write_RealMatrix(M1^);
     Write_Message('Another random real matrix:');
     Random_RealMatrix(M2^);
     Write_RealMatrix(M2^);
     if Sender = Addition1 then
     begin
     Write_Message('Sum of both:');
     RMatrix_Addition(MResult^, M1^, M2^);
     Write_RealMatrix(MResult^);
     end
     else if Sender = Subtraction1 then
     begin
     Write_Message('First - second:');
     RMatrix_Subtraction(MResult^, M1^, M2^);
     Write_RealMatrix(MResult^);
     end
     else if Sender = Multiplication1 then
     begin
     Write_Message('First * second:');
     RMatrix_Multiplication(MResult^, M1^, M2^);
     Write_RealMatrix(MResult^);
     end
     else if Sender = ToPower1 then
     begin
     i:= random(10);
     Write_Message('First matrix to power ' + IntToStr(i) + ':');
     RMatrix_ToPower(M1^, i);
     Write_RealMatrix(M1^);
     end;
     finally
     dispose(M1);
     dispose(M2);
     dispose(MResult);
     end;
end;

{ Complex counterpart of above. }
procedure TMainForm.ComplexArithmetic(Sender: TObject);
var M1, M2, MResult: ^ComplexMatrix;
    i: integer;
begin
     new(M1);
     new(M2);
     new(MResult);
     try
     Write_Message('A random complex matrix:');
     Random_ComplexMatrix(M1^);
     Write_ComplexMatrix(M1^);
     Write_Message('Another random complex matrix:');
     Random_ComplexMatrix(M2^);
     Write_ComplexMatrix(M2^);
     if Sender = Addition2 then
     begin
     Write_Message('Sum of both:');
     CMatrix_Addition(MResult^, M1^, M2^);
     Write_ComplexMatrix(MResult^);
     end
     else if Sender = Subtraction2 then
     begin
     Write_Message('First - second:');
     CMatrix_Subtraction(MResult^, M1^, M2^);
     Write_ComplexMatrix(MResult^);
     end
     else if Sender = Multiplication2 then
     begin
     Write_Message('First * second:');
     CMatrix_Multiplication(MResult^, M1^, M2^);
     Write_ComplexMatrix(MResult^);
     end
     else if Sender = ToPower2 then
     begin
     i:= random(10);
     Write_Message('First matrix to power ' + IntToStr(i) + ':');
     CMatrix_ToPower(M1^, i);
     Write_ComplexMatrix(M1^);
     end;
     finally
     dispose(M1);
     dispose(M2);
     dispose(MResult);
     end;
end;

{ Functions for real matrices. }
procedure TMainForm.RealFunction(Sender: TObject);
var M, M1: ^RealMatrix;
    Rank: integer;
    Trace, Determinant, x: extended;
begin
     new(M);
     new(M1);
     try
     Write_Message('A random real matrix:');
     Random_RealMatrix(M^);
     Write_RealMatrix(M^);
     if Sender = Trace1 then
     begin
     Trace:=  RMatrix_Trace(M^);
     Write_Message('Trace of matrix: ' + FloatToStr(Trace));
     end
     else if Sender = Determinant1 then
     begin
     Determinant:= RMatrix_Determinant(M^);
     Write_Message('Determinant of matrix: ' + FloatToStr(Determinant));
     end
     else if Sender = Transpose1 then
     begin
     Write_Message('Transposed matrix:');
     RMatrix_Transpose(M^);
     Write_RealMatrix(M^);
     end
     else if Sender = Rank1 then
     begin
     Rank:= RMatrix_Rank(M^);
     Write_Message('Rank of matrix: ' + IntToStr(Rank));
     end
     else if Sender = Inverse1 then
     begin
     { Some people asked me about the accuracy of the algorithms.
       Here you can test the accuracy of the inversion algorithm.
       TestInverse will calculate (Original matrix) * (Inverse matrix) and check the result
       with an identity matrix.
       The result is the greatest discrepancy between both matrices.
       (lower numbers mean better result)
       Note the 'RCopy_Matrix(...) line.
       This will copy a matrix (including its dimensions) to another. }
     RCopy_Matrix(M^, M1^);
     Write_Message('Inverse of matrix:');
     RMatrix_Inverse(M^);
     Write_RealMatrix(M^);
     x:= RMatrix_TestInverse(M^, M1^);
     Write_Message('Greatest discrepancy in inverse matrix: ' + FloatToStr(x));
     end;
     finally
     dispose(M);
     dispose(M1);
     end;
end;

{ Complex counterpart of above. }
procedure TMainForm.ComplexFunction(Sender: TObject);
var M: ^ComplexMatrix;
    Rank: integer;
    Trace, Determinant: complex;
begin
     new(M);
     try
     Write_Message('A random Complex matrix:');
     Random_ComplexMatrix(M^);
     Write_ComplexMatrix(M^);
     if Sender = Trace2 then
     begin
     Trace:=  CMatrix_Trace(M^);
     Write_Message('Trace of matrix: ' + '(' + FloatToStr(Trace.re) + ';' + FloatToStr(Trace.im) + ')');
     end
     else if Sender = Determinant2 then
     begin
     Determinant:= CMatrix_Determinant(M^);
     Write_Message('Determinant of matrix: ' + '(' + FloatToStr(Determinant.re) + ';' +
     FloatToStr(Determinant.im) + ')');
     end
     else if Sender = Transpose2 then
     begin
     Write_Message('Transposed matrix:');
     CMatrix_Transpose(M^);
     Write_ComplexMatrix(M^);
     end
     else if Sender = Rank2 then
     begin
     Rank:= CMatrix_Rank(M^);
     Write_Message('Rank of matrix: ' + IntToStr(Rank));
     end
     else if Sender = Inverse2 then
     begin
     Write_Message('Inverse of matrix:');
     CMatrix_Inverse(M^);
     Write_ComplexMatrix(M^);
     { Eventually test the inverse matrix... see above. }
     end;
     finally
     dispose(M);
     end;
end;

{ Reduce a real matrix to its Gauss/Hermite normal form. }
procedure TMainForm.RealReduction(Sender: TObject);
var M: ^RealMatrix;
begin
     new(M);
     try
     Write_Message('A random real matrix:');
     Random_RealMatrix(M^);
     Write_RealMatrix(M^);
     if Sender = Gauss1 then
     begin
     Write_Message('Matrix reduced to Gauss Normal form:');
     RMatrix_GaussNormal(M^);
     Write_RealMatrix(M^);
     end
     else if Sender = Hermite1 then
     begin
     Write_Message('Matrix reduced to hermite normal form:');
     RMatrix_HermiteNormal(M^);
     Write_RealMatrix(M^);
     end;
     finally
     dispose(M);
     end;
end;

{ Complex counterpart of above. }
procedure TMainForm.ComplexReduction(Sender: TObject);
var M: ^ComplexMatrix;
begin
     new(M);
     try
     Write_Message('A random real matrix:');
     Random_ComplexMatrix(M^);
     Write_ComplexMatrix(M^);
     if Sender = Gauss2 then
     begin
     Write_Message('Matrix reduced to Gauss Normal form:');
     CMatrix_GaussNormal(M^);
     Write_ComplexMatrix(M^);
     end
     else if Sender = Hermite2 then
     begin
     Write_Message('Matrix reduced to hermite normal form:');
     CMatrix_HermiteNormal(M^);
     Write_ComplexMatrix(M^);
     end;
     finally
     dispose(M);
     end;
end;

{ Matrix creation functions.
  Here are them for a real matrix.
  The complex counterparts are almost identical, implement these yourself. }
procedure TMainForm.CreateRealMatrix(Sender: TObject);
var M: ^RealMatrix;
    i, j: integer;
    V: ^RealVector;
begin
     new(M);
     new(V);
     try
     Write_Message('A random real matrix:');
     Random_RealMatrix(M^);
     Write_RealMatrix(M^);
     i:= random(5);
     j:= random(5);
     if Sender = AddRow1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Vector added (as a row) to matrix:');
     RMatrixAddRow(M^, V^);
     Write_RealMatrix(M^);
     end
     else if Sender = AddColumn1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Vector added (as a column) to matrix:');
     RMatrixAddColumn(M^, V^);
     Write_RealMatrix(M^);
     end
     else if Sender = DeleteRow1 then
     begin
     Write_Message(IntToStr(i) + '-th row deleted in matrix:');
     RMatrixDeleteRow(M^, i);
     Write_RealMatrix(M^);
     end
     else if Sender = DeleteColumn1 then
     begin
     Write_Message(IntToStr(i) + '-th column deleted in matrix:');
     RMatrixDeleteColumn(M^, i);
     Write_RealMatrix(M^);
     end
     else if Sender = SwapRows1 then
     begin
     Write_Message('The ' + IntToStr(i) + '-th row swapped with the ' + IntToStr(j) + '-th:');
     RSwap_Rows(M^, i, j);
     Write_RealMatrix(M^);
     end
     else if Sender = SwapColumns1 then
     begin
     Write_Message('The ' + IntToStr(i) + '-th column swapped with the ' + IntToStr(j) + '-th:');
     RSwap_Columns(M^, i, j);
     Write_RealMatrix(M^);
     end
     else if Sender = InsertRow1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Vector inserted at ' + IntToStr(i) + '-th position (as a row) in matrix:');
     RMatrixInsertRow(M^, V^, i);
     Write_RealMatrix(M^);
     end
     else if Sender = InsertColumn1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Vector inserted at ' + IntToStr(i) + '-th position (as a row) in matrix:');
     RMatrixInsertColumn(M^, V^, i);
     Write_RealMatrix(M^);
     end
     else if Sender = ReplaceRow1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Vector inserted at ' + IntToStr(i) + '-th position (as a row) in matrix:');
     RMatrixReplaceRow(M^, V^, i);
     Write_RealMatrix(M^);
     end
     else if Sender = ReplaceColumn1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Vector inserted at ' + IntToStr(i) + '-th position (as a column) in matrix:');
     RMatrixReplaceColumn(M^, V^, i);
     Write_RealMatrix(M^);
     end
     else if Sender = MatrixVector1 then
     begin
     Write_Message('Matrix transformed to a vector:');
     RMatrixToVector(V^, M^);
     Write_RealVector(V^);
     end
     else if Sender = VectorMatrix1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Vector transformed to a matrix:');
     RVectorToMatrix(M^, V^);
     Write_RealMatrix(M^);
     end
     else if Sender = MatrixRowVector1 then
     begin
     Write_Message('Matrix row ' + IntToStr(i) + ' transformed to a vector:');
     RMatrixRowToVector(V^, M^, i);
     Write_RealVector(V^);
     end
     else if Sender = MatrixColVector1 then
     begin
     Write_Message('Matrix column ' + IntToStr(i) + ' transformed to a vector:');
     RMatrixColToVector(V^, M^, i);
     Write_RealVector(V^);
     end
     else if Sender = ReplaceMatrixElement1 then
     begin
     Write_Message('Replaced element ' + IntToStr(i) + '-' + IntToStr(j) + ' with its square:');
     RMatrixReplaceElement(M^, i, j, sqr(M^.Matrix[i,j]));
     Write_RealMatrix(M^);
     end
     finally
     dispose(M);
     dispose(V);
     end;
end;

{ Special real matrices.
  Some of them may seem obscure for some readers.
  If you don't know the definitions, you may never have to use them.
  (You can, of course, look them up in a book.)  }
procedure TMainForm.SpecialRealMatrix(Sender: TObject);
var M: ^RealMatrix;
    i: integer;
    V: ^RealVector;
begin
     new(M);
     new(V);
     try
     i:= random(5);
     if Sender = Identity1 then
     begin
     Write_Message('Identity matrix of order ' + IntToStr(i) + ':');
     RIdentity_Matrix(M^, i);
     Write_RealMatrix(M^);
     end
     else if Sender = Hilbert1 then
     begin
     Write_Message('Hilbert matrix of order ' + IntToStr(i) + ':');
     RHilbert_Matrix(M^, i);
     Write_RealMatrix(M^);
     end
     else if Sender = Vandermonde1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Vandermonde matrix created using this vector:');
     RVandermonde_Matrix(M^, V^);
     Write_RealMatrix(M^);
     end
     else if Sender = Householder1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(V^);
     Write_RealVector(V^);
     Write_Message('Householder (transformation) matrix created using this vector:');
     RHouseholder_Matrix(M^, V^);
     Write_RealMatrix(M^);
     end
     finally
     dispose(M);
     dispose(V);
     end;
end;

{ Complex counterpart of above. }
procedure TMainForm.SpecialComplexMatrix(Sender: TObject);
var M, U: ^ComplexMatrix;
    i: integer;
    V: ^ComplexVector;
begin
     new(M);
     New(U);
     new(V);
     try
     i:= random(5);
     if Sender = Identity2 then
     begin
     Write_Message('Identity matrix of order ' + IntToStr(i) + ':');
     CIdentity_Matrix(M^, i);
     Write_ComplexMatrix(M^);
     end
     else if Sender = Hilbert2 then
     begin
     Write_Message('Hilbert matrix of order ' + IntToStr(i) + ':');
     CHilbert_Matrix(M^, i);
     Write_ComplexMatrix(M^);
     end
     else if Sender = Vandermonde2 then
     begin
     Write_Message('A random complex vector:');
     Random_ComplexVector(V^);
     Write_ComplexVector(V^);
     Write_Message('Vandermonde matrix created using this vector:');
     CVandermonde_Matrix(M^, V^);
     Write_ComplexMatrix(M^);
     end
     else if Sender = Householder2 then
     begin
     Write_Message('A random complex vector:');
     Random_ComplexVector(V^);
     Write_ComplexVector(V^);
     Write_Message('Householder (transformation) matrix created using this vector:');
     CHouseholder_Matrix(M^, V^);
     Write_ComplexMatrix(M^);
     end
     else if Sender = Fourier1 then
     begin
     Write_Message('Fourier matrices of order ' + IntToStr(i) + ':');
     Fourier_Matrix(M^, U^, i);
     Write_ComplexMatrix(M^);
     Write_Message(' ');
     Write_ComplexMatrix(U^);
     end
     finally
     dispose(M);
     dispose(U);
     dispose(V);
     end;
end;

{ Factorize a real matrix in its L & U (Q & R) counterparts.
  We've also added the function to orthogonalize a set of (linearly indepent) vectors
  using the Gram-Schmidt theorem.
  The vectors are given as the columns of a matrix.
  An orthogonal set of vectors is a set of vectors where every vectors is perpendicular
  with respect to the others and has unit length (1).                              }
procedure TMainForm.RealMatrixFactorization(Sender: TObject);
var M, M1, M2: ^RealMatrix;
    x: extended;
begin
     new(M);
     new(M1);
     new(M2);
     try
     Write_Message('A random real matrix:');
     Random_RealMatrix(M^);
     Write_RealMatrix(M^);
     if Sender = LU1 then
     begin
     Write_Message('Matrix factorized in L & U counterparts:');
     RMatrix_LU(M^, M1^, M2^);
     Write_RealMatrix(M1^);
     Write_Message(' ');
     Write_RealMatrix(M2^);
     x:= RMatrix_TestLU(M^, M1^, M2^);
     Write_Message('Greatest discrepancy in LU-factorization of matrix: ' + FloatToStr(x));
     end
     else if Sender = OrthogonalizeSetOfVectors1 then
     begin
     Write_Message('Orthogonalizing set of vectors (the columns of the matrix:');
     RMatrix_GramSchmidt(M^);
     Write_RealMatrix(M^);
     x:= RMatrix_TestOrthogonality(M^);
     Write_Message('Greatest discrepancy in orthogonalization of vectors: ' + FloatToStr(x));
     end;
     finally
     dispose(M);
     dispose(M1);
     dispose(M2);
     end;
end;

procedure TMainForm.CharacteristicEquation1Click(Sender: TObject);
var M: ^RealMatrix;
    V: ^RealVector;
begin
    New(M);
    New(V);
    try
    Write_Message('A random real matrix:');
    Random_RealMatrix(M^);
    Write_RealMatrix(M^);
    { Write cofficients of characteristic equation to screen...  }
    Write_Message('Coefficients of characteristic polynomial');
    RMatrix_CharEq(M^, V^);
    Write_RealVector(V^);
    finally
    Dispose(M);
    Dispose(V);
    end;
end;

{ Solve a system of (linear) equations.
  Note that the result is given in the form of a matrix.
  Every row in this matrix represents a distinctive solution for the system.
  The first row is the 'main' solution.
  If the matrix has more than one row, the general solution for the system is
  Row1 + a*Row2 + b*Row3 + etc...
  a, b, etc. any real number.                                                }
procedure TMainForm.SysofEquations1Click(Sender: TObject);
var M, MC: ^RealMatrix;
    V: ^RealVector;
    HasSolution: boolean;
begin
   new(M);
   new(MC);
   new(V);
   try
   MC^.Row:= 4;
   MC^.Col:= 4;
   V^.Dimension:= 4;
   V^.Vector[1]:= 37;
   V^.Vector[2]:= 6;
   V^.Vector[3]:= 75;
   V^.Vector[4]:= 7;
   MC^.Matrix[1,1]:= 6;
   MC^.Matrix[1,2]:= 1;
   MC^.Matrix[1,3]:= -3;
   MC^.Matrix[1,4]:= 0;
   MC^.Matrix[2,1]:= -2;
   MC^.Matrix[2,2]:= 3;
   MC^.Matrix[2,3]:= 5;
   MC^.Matrix[2,4]:= -7;
   MC^.Matrix[3,1]:= 8;
   MC^.Matrix[3,2]:= 0;
   MC^.Matrix[3,3]:= 4;
   MC^.Matrix[3,4]:= -5;
   MC^.Matrix[4,1]:= 0;
   MC^.Matrix[4,2]:= -7;
   MC^.Matrix[4,3]:= -4;
   MC^.Matrix[4,4]:= 1;
   Write_Message('System of linear equations A*X = Y:');
   Write_Message('Cofficient matrix A of system:');
   Write_RealMatrix(MC^);
   Write_Message('Vector Y of system:');
   Write_RealVector(V^);
   SysOfEquations(M^, MC^, V^, HasSolution);
   if HasSolution = false then
   Write_Message('System has no solution!') else
   begin
   Write_Message('Solution(s) for system:');
   Write_RealMatrix(M^);
   end;
   finally
   dispose(M);
   dispose(MC);
   dispose(V);
   end;
end;

procedure TMainForm.RealVector1Click(Sender: TObject);
var RV: ^RealVector;
    CV: ^ComplexVector;
begin
     new(RV);
     new(CV);
     try
     if Sender = RealVector1 then
     begin
     Write_Message('A random real vector:');
     Random_RealVector(RV^);
     Write_RealVector(RV^);
     end
     else
     begin
     Write_Message('A random complex vector:');
     Random_ComplexVector(CV^);
     Write_ComplexVector(CV^);
     end;
     finally
     dispose(RV);
     dispose(CV);
     end;
end;

procedure TMainForm.Norm1Click(Sender: TObject);
var RV: ^RealVector;
    x: extended;
begin
     new(RV);
     try
     Write_Message('A random real vector:');
     Random_RealVector(RV^);
     Write_RealVector(RV^);
     if Sender = Norm1 then
     begin
     x:= RVector_Norm(RV^);
     Write_Message('Norm of vector: ' + FloatToStr(x));
     end
     else
     begin
     Write_Message('A unit vector (in the same direction):');
     RUnit_Vector(RV^);
     Write_RealVector(RV^);
     end;
     finally
     dispose(RV);
     end;
end;

procedure TMainForm.Norm2Click(Sender: TObject);
var CV: ^ComplexVector;
    c: complex;
begin
     new(CV);
     try
     Write_Message('A random complex vector:');
     Random_ComplexVector(CV^);
     Write_ComplexVector(CV^);
     if Sender = Norm2 then
     begin
     c:= CVector_Norm(CV^);
     Write_Message('Norm of vector: (' + FloatToStr(c.re) + ';' + FloatToStr(c.im) + ')');
     end
     else
     begin
     Write_Message('A unit vector (in the same direction):');
     CUnit_Vector(CV^);
     Write_ComplexVector(CV^);
     end;
     finally
     dispose(CV);
     end;
end;

{ Add, subtract two vectors.
  Find the scalar (dot) and vector (cross) product of two vectors.
  Calculate the angle and distance between two vectors.
  Note that Angle is in radians. By using RadToDeg (in calculus.dcu) we set this to degrees
  ( to set this to degrees Fahrenheit or Kelvin we use ... ;))  }
procedure TMainForm.RealVectorFunction(Sender: TObject);
var V, V1, V2: ^RealVector;
    x: extended;
begin
     new(V1);
     new(V2);
     new(V);
     try
     Write_Message('A random real vector:');
     Random_RealVector(V1^);
     Write_RealVector(V1^);
     Write_Message('Another random real vector:');
     Random_RealVector(V2^);
     Write_RealVector(V2^);
     if Sender = Addition3 then
     begin
     Write_Message('Sum of both vectors:');
     RVector_Addition(V^, V1^, V2^);
     Write_RealVector(V^);
     end
     else if Sender = Subtraction3 then
     begin
     Write_Message('First vector minus the second:');
     RVector_Subtraction(V^, V1^, V2^);
     Write_RealVector(V^);
     end
     else if Sender = ScalarDotProduct1 then
     begin
     x:= RVector_DotProduct(V1^, V2^);
     Write_Message('Scalar (dot) product of vectors: ' + FloatToStr(x));
     end
     else if Sender = VectorCrossProduct1 then
     begin
     Write_Message('Vector (cross) product of both:');
     RVector_CrossProduct(V^, V1^, V2^);
     Write_RealVector(V^);
     end
     else if Sender = Angle1 then
     begin
     x:= RVector_Angle(V1^, V2^);
     Write_Message('Angle between 2 vectors: ' + FloatToStr(RadToDeg(x)));
     end
     else if Sender = Distance1 then
     begin
     x:= RVector_Distance(V1^, V2^);
     Write_Message('Distance between 2 vectors: ' + FloatToStr(x));
     end
     finally
     dispose(V);
     dispose(V1);
     dispose(V2);
     end;
end;

end.
